## importing libs
import pandas as pd
import json
from sqlalchemy import create_engine
import psycopg2

def get_data(query, engine, close_connection = False):
    """
    make a query in a database
    """
    ## open connection
    engine.connect()
    
    ## make select query
    table = pd.read_sql(query, engine)

    # ## close connection
    if close_connection:
        engine.close()
    return table

def write_table(table, table_name, schema, engine, if_exists = 'append', chunksize = 10_000, index= False, close_connection = False):
    """
    if_exists => ["append","replace"]
    
    """

    ## open connection
    engine.connect()

    ## write or replace table
    table.to_sql(name = table_name,
                 if_exists = if_exists,
                 con = engine,
                 schema=schema,
                 method='multi',
                 chunksize = chunksize,
                 index = index)

    # ## close connection
    if close_connection:
        engine.close()

def get_first_row(table, schema, engine):
    return get_data(f'select * from {schema}.{table} dc limit 1',engine)

def create_table_structure(pandas_df, table_name, engine, schema, index = False, if_exists = 'append'):
    pandas_df.to_sql(name = table_name,
                    con = engine, 
                    schema=schema, index=index, if_exists = if_exists)
    conn = engine.connect()
    conn.execute(f'DELETE FROM {schema}.{table_name}')
    conn.close()


import os, json

def get_files(folder, contains = 'INMET_SE_ES_'):
    folder = './{folder}'.format(folder = folder)
    return list(filter(lambda k: str(contains) in k, os.listdir(folder)))

def open_json(json_file):
    with open(json_file) as data:
        return json.load(data)


import pandas as pd

def crossjoin(column_1,column_2):
    """
    Create a crossjoin dataframe based on 2 columns
    """
    columns_name = [column_1.name,column_2.name]
    column_1 = pd.DataFrame(column_1.unique())
    column_2 = pd.DataFrame(column_2.unique())
    column_1['key'], column_2['key'] = 0,0
    df_ret = column_1.merge(column_2,on='key').drop(columns = 'key')
    df_ret.columns = columns_name
    return df_ret
    


import dask as dd

def send_table(input_table, uri_input, output_table, uri_output, schema, parallel = True, method='multi', chunksize = 50_000, if_exists = 'append'):
    df = dd.read_sql(table = input_table, uri = uri_input)
    dd.to_sql(df,uri=uri_output, name=output_table, schema=schema, parallel=parallel, method='multi',chunksize=chunksize,if_exists=if_exists)


def migrate(uri_origin, uri_destiny, destiny_schema):
    def output_func(table_origin, table_destiny, index_column, parallel=True, method='multi',chunksize = 30_000, if_exists='replace'):
        df = dd.read_sql(table = table_origin, uri = uri_origin, index_column = index_column)
        dd.to_sql(df,uri=uri_destiny, name=table_destiny, schema=destiny_schema, parallel=parallel, method='multi',chunksize=chunksize,if_exists=if_exists)    
    return output_func


from datetime import datetime as dt

class Timer():    
    """    
    Timer class
    Declare a timer object that measure the time 
    of a running application
    """    
    def __init__(self, start_now = False):
        if start_now:
            self.start()

    def start(self):
        if hasattr(self,'start_time'):
            print('Started at: {}'.format(self.start_time))
        else:
            self.start_time = dt.now()
    
    def stop(self):
        if hasattr(self,'stop_time'):
            print('Stopped at: {}'.format(self.stop_time))
        else:
            self.stop_time = dt.now()
            self.duration = self.stop_time - self.start_time


import pandas as pd
import numpy as np
from jh_utils.time_series.covariables import year_sincos_array, hour_sincos_array, hours_by_day
from jh_utils.pandas.preprocessing import make_dummies

def date_range_dataframe(start_date,end_date,freq='1H', column_name='hours'):
    df = pd.DataFrame({column_name: pd.date_range(start_date, end_date, freq=freq, closed='left')})
    return df

def time_series_dataframe(start_date,end_date,freq='1H',datetime_column_name='hours',
                          weekday_dummies=True,
                          month_dummies=True,
                          hour_dummies=True,
                          year_sin_cos=True,
                          hour_sin_cos=True):
    df = date_range_dataframe(start_date,end_date,freq=freq,column_name=datetime_column_name)
    shape = df.shape
    if weekday_dummies:
        df = pd.concat([df,make_dummies(pd.Series(df.iloc[:,0].dt.weekday, name = 'weekday_dummie'))],axis=1)
    if month_dummies:
        df = pd.concat([df,make_dummies(pd.Series(df.iloc[:,0].dt.month, name = 'month_dummie'))],axis=1)
    if hour_dummies:
        df = pd.concat([df,make_dummies(pd.Series(df.iloc[:,0].dt.hour, name = 'hour_dummie'))],axis=1)
    if year_sin_cos:
        df = pd.concat([df,year_sincos_array(shape[0],start_date,end_date)],axis=1)
    if hour_sin_cos:
        df = pd.concat([df,hour_sincos_array(hours_by_day(freq),start_date,pd.to_datetime(end_date))],axis=1)
    
    df.index =  df[datetime_column_name]
    df = df.iloc[:,1:]
    return df

def date_hour_crossjoin(column_1,column_2):
    """
    Create a crossjoin dataframe based on 2 columns
    """
    column_1 = pd.DataFrame(column_1.unique())
    column_2 = pd.DataFrame(column_2.unique())
    column_1['key'], column_2['key'] = 0,0
    date_time = column_1.merge(column_2,on='key').drop(columns = 'key')
    date_time.columns = ['data','hora']
    date_time.sort_values(['data','hora'],inplace=True)
    date_time.reset_index(inplace=True,drop=True)
    return date_time


from jh_utils.pandas import sql

def send_data(engine_origin, engine_destiny, destiny_schema, index=False):
    def output_func(query, table_name, if_exists = 'replace'):
        df = sql.get_data(query, engine_origin)
        sql.write_table(df, table_name, destiny_schema, engine_destiny, if_exists = if_exists, chunksize = 10_000, index= False, close_connection = False)
    return output_func


import pandas as pd
import numpy as np

def hours_by_day(freq):
    freq = freq.upper()
    if freq == '1H':
        return 24
    if freq == '2H':
        return 12
    if freq == '3H':
        return 8
    if freq == '4H':
        return 6
    if freq == '6H':
        return 4
    if freq == '8H':
        return 3
    if freq == '12H':
        return 2

def year_sincos_array(n,start_date,end_date):
    start_date = pd.to_datetime(start_date)
    end_date   = pd.to_datetime(end_date)
    
    ## day distance
    first_date = pd.to_datetime('{year}-01-01'.format(year = start_date.year))
    day_of_the_year_start = (start_date - first_date).days
        
    ## year distance
    distance = (end_date - first_date).days
    
    ##
    start = (np.pi*2)*(day_of_the_year_start/365)
    end = np.pi*2*(distance/365)
    
    length = np.arange(start, end, (end-start)/n)
    df =  pd.concat([pd.Series(np.sin(length)),pd.Series(np.cos(length))],axis=1)
    df.columns = ['sin_year','cos_year']
    return df

def hour_sincos_array(hours_by_day, start_date, end_date):
    start_date = pd.to_datetime(start_date)
    end_date   = pd.to_datetime(end_date)
    ## ! year distance
    distance = (end_date - start_date).days

    start = 0
    end = np.pi*2*distance
    length = np.arange(start, end, np.pi*2/(hours_by_day))
    df =  pd.concat([pd.Series(np.sin(length)),pd.Series(np.cos(length))],axis=1)
    df.columns = ['sin_day','cos_day']
    return df.iloc[:-1]